/*
 * imrows.c -- Extract a sum of rows from a fits ccd 2D image
 *
 * John Kielkopf
 * kielkopf@louisville.edu
 * 
 *
 * Current Version 1.0 -- March 5, 2005
 *
 *   March 5, 2005 : Version 1.0 
 *   
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "fitsio.h"

int main(int argc, char *argv[])
{
    fitsfile *afptr;  
    FILE *fpdata;   
    int status = 0;  
    int atype, anaxis, check = 1, ii;
    int startrow, stoprow, totpix, icol;
    long npixels = 1, firstpix[2] = {1,1};
    long anaxes[2] = {1,1};
    double *apix, *sum;

    if (argc != 5) { 
      printf("Usage: imrows in_image out_data start_row stop_row\n");
      printf("\n");
      printf("Write a data file with  a sum of fits image rows\n");
      printf("\n");
      printf("Example: \n");
      printf("  imrows image.fits rows.dat 100 200\n");
      return(0);
    }
    
    startrow = atoi(argv[3]);
    stoprow = atoi(argv[4]);

    /* Open input image */
    
    fits_open_file(&afptr, argv[1], READONLY, &status); 

    /* Read dimensions */

    fits_get_img_dim(afptr, &anaxis, &status);  
    fits_get_img_size(afptr, 2, anaxes, &status);

    /* Report errors if any */

    if (status) {
       fits_report_error(stderr, status); 
       return(status);
    }

    /* Check the image sizes */
    
    if (anaxis != 2) 
    {
       printf("Error: only a 2-D image is supported\n");
       fits_close_file(afptr, &status);
       return(1);    
    }
    
    /* Check the row range */
    
    if (stoprow < startrow) 
    {
       printf("Initial row must be less than or equal to last row\n");
       fits_close_file(afptr, &status);
       return(1);    
    }
    
    if (startrow < 1) 
    {
       printf("Initial row must be greater than 1\n");
       fits_close_file(afptr, &status);
       return(1);    
    }

    if (stoprow > anaxes[1]) 
    {
       printf("Last row must be less than %d\n for this image", anaxes[1]);
       fits_close_file(afptr, &status);
       return(1);    
    }


    /* Open a file to write the sum data */

    fpdata = fopen(argv[2],"w");  
  
    /* Number of pixels to read in each row */
    
    if (fpdata == NULL ) 
    { 
      printf("Unable to open output file for data\n");
      fits_close_file(afptr, &status);
      return(1);
    }
    
    npixels = anaxes[0];
    
    /* Allocate memory for one row */ 

    apix = (double *) malloc(npixels * sizeof(double)); 

    if (apix == NULL ) 
    {
      printf("Image row memory allocation error\n");
      return(1);
    }


    /* Allocate memory for the sum */ 

    sum = (double *) malloc(npixels * sizeof(double)); 

    if (sum == NULL ) 
    {
      printf("Sum data memory allocation error\n");
      return(1);
    }
    
    /* Initialize the sum array */
    
    for (ii = 0; ii < anaxes[0]; ii++) 
    {
      sum[ii] = 0.;                      
    }         

    /* Check size of image */
    
    totpix = anaxes[0] * anaxes[1];
    if (totpix == 0)
    {
      printf("The  image is empty\n");
      return(1);
    }
    
    /* Start with first pixel in each row */
    
    firstpix[0] = 1;  

    /* Process image one row at a time from startrow to stoprow */
    
/*
    for (firstpix[1] = 1; firstpix[1] <= anaxes[1]; firstpix[1]++)
*/
    for (firstpix[1] = startrow; firstpix[1] <= stoprow; firstpix[1]++)
    {  

      /* Give starting pixel coordinate and number of pixels to read */
       
      if (fits_read_pix(afptr, TDOUBLE, firstpix, npixels,
           NULL, apix, NULL, &status))
        break;   

      for (ii = 0; ii < anaxes[0]; ii++) 
      {
        sum[ii] += apix[ii];                      
      }
    }
    
    /* Write the data file */
    
    for (ii = 0; ii < anaxes[0]; ii++) 
    {      
      icol = ii + 1;
      fprintf(fpdata, "%d  %lf\n", icol, sum[ii]);   
    }
    
    
    free(apix);
    free(sum);

    fits_close_file(afptr, &status);
    fclose(fpdata);

    /* Report any error message */
 
    if (status) fits_report_error(stderr, status); 
    return(status);
}

